
	;For production
	;.equ	location, 0x8000
	;.equ    buffer_location, 0xA000

	;For code development
	.equ	location, 0x2000
	.equ    buffer_location, 0x4000

        ;.equ    chip_size, 0x4000
        .equ    chip_size, 0x3000

	.equ	fuse_msb, 11011001b
	.equ	fuse_lsb, 11101111b
	;.equ	fuse_lsb, 11100001b

	.equ	baud_const, 255		;115200 baud w/ 22.1182 MHz

	.equ	buf_en_pin, 0x90
	.equ	sclk_pin, 0x91
	.equ	rst_pin, 0x92
	.equ	mosi_pin, 0x93
	.equ	miso_pin, 0x94
	.equ	txd_en_pin, 0x95



.equ    cin, 0x0032
.equ    cout, 0x0030
.equ    phex, 0x0034
.equ    pstr, 0x0038
.equ    newline, 0x0048


.equ    sig_byte_1, 0x1E
.equ    sig_byte_2, 0x94
.equ    sig_byte_3, 0x03



.org    location
.db     0xA5,0xE5,0xE0,0xA5     ;signiture bytes
.db     254,'P',0,0             ;id (254=cmd)
.db     0,0,0,0                 ;prompt code vector
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;user defined
.db     255,255,255,255         ;length and checksum (255=unused)
.db     "Atmel AVR Mega16 Programmer",0
.org    location+64             ;executable code begins here


begin:
	mov	r7, th1			;hold baud in r7
	mov	r6, #8			;r6 = reboot retry counter
reboot:
        mov     dptr, #msg_reset
        lcall   pstr
	acall	enter_pgm_mode

restart:
	acall	delay_3ms
	acall	programming_enable
	cjne	a, #0x53, startup_error
	clr	a
	acall	read_signature_byte
	cjne	a, #sig_byte_1, device_error
	mov	a, #1
	acall	read_signature_byte
	cjne	a, #sig_byte_2, device_error
	mov	a, #2
	acall	read_signature_byte
	cjne	a, #sig_byte_3, device_error
	sjmp	menu

startup_error:
	;if we didn't get the correct response, it's likely that the
	;AVR heard some noise at is 'off by 1 bit'... eg, the 0x53 will
	;come back 0xA6 if it's one clock out of sync.  Saddly, there
	;is no way the AVR gets a reliable byte framing signal like a
	;start/stop bit, so this startup check is critical to make sure
	;it is properly synced to the byte boundry
	;jnb	ri, running_loop_1200
	djnz	r6, reboot		;retry a few times
	push	acc
	mov	dptr, #msg_startup_error
	lcall	pstr
	pop	acc
	lcall	phex
	lcall	newline
	mov	dptr, #msg_startup_error_2
	lcall	pstr
	ret

device_error:
	mov	dptr, #msg_device_error
	lcall	pstr
	ret


menu:
        mov     dptr, #mesg_menu
        lcall   pstr
        lcall   cin
	cjne	a, #':', menu0
	lcall	0x2B8			;PM2 download
	sjmp	menu
menu0:  cjne    a, #'r', menu1          ;Read Chip
        acall   read_chip
        sjmp    menu
menu1:  cjne    a, #'w', menu2          ;Write Chip
        acall   erase_chip
        acall   write_chip
        acall   verify_chip
	acall	set_fuses
        ajmp    menu
menu2:  cjne    a, #'v', menu3          ;Verify
        acall   verify_chip
	mov	dptr, #msg_ck_fuses
	lcall	pstr
	acall	fuses_verify
        ajmp    menu
menu3:  cjne    a, #'e', menu4          ;Erase
        acall   erase_chip
        ajmp    menu
menu4:  cjne    a, #'q', menu5          ;Quit
	setb	txd_en_pin
	setb	buf_en_pin
        lcall   newline
        ret
menu5:	cjne	a, #'b', menu6
	acall	boot_chip
	ajmp	restart
menu6:
        ajmp    menu



;boot_chip:
;	mov	dptr, #msg_boot
;	lcall	pstr
;	jnb	ti, *
;	acall	delay_10ms
;	setb	en_pin
;running_loop_115200:
;	jnb	p3.2, running_115200_halt
;	lcall	cin
;	add	a, #256 - 27
;	jnz	running_loop_115200
;running_115200_halt:
;	;halt chip and reenter programming mode
;	acall	enter_pgm_mode
;	acall	delay_10ms
;	ret


boot_chip:
	mov	dptr, #msg_boot
	lcall	pstr
	jnb	ti, *
	setb	buf_en_pin
	acall	delay_10ms
	clr	txd_en_pin
	acall	delay_3ms
	setb	miso_pin
	setb	mosi_pin
	setb	sclk_pin
	setb	rst_pin
	acall	delay_10ms
running_loop:
	lcall	cin
	cjne	a, #':',running_check_esc
	setb	txd_en_pin
	clr	sclk_pin
	clr	rst_pin
	lcall	0x2B8			;intel hex download
	acall	enter_pgm_mode
	acall	programming_enable
	cjne	a, #0x53, running_halt
	clr	a
	acall	read_signature_byte
	cjne	a, #sig_byte_1, running_halt
	mov	a, #1
	acall	read_signature_byte
	cjne	a, #sig_byte_2, running_halt
	mov	a, #2
	acall	read_signature_byte
	cjne	a, #sig_byte_3, running_halt
	lcall	erase_chip
	acall	write_chip
	acall	set_fuses
	;lcall	copy_protect
	sjmp	boot_chip
running_check_esc:
	cjne	a, #27, running_loop
running_halt:
	acall	enter_pgm_mode
	ret






write_chip:
	mov	dptr, #msg_writing
	lcall	pstr
	mov	dptr, #buffer_location
	mov	r2, #0		;r2 to count page number
write_chip_page_loop:
	acall	is_data_to_write
	jnc	write_chip_skip_page
	mov	r4, #0		;r4 to count page offset
write_chip_word_loop:
	movx	a, @dptr
	acall	wr_lsb
	inc	dptr
	movx	a, @dptr
	acall	wr_msb
	inc	dptr
	inc	r4
	cjne	r4, #0x40, write_chip_word_loop
	;now that we've loaded all 128 bytes, it's time to write this page
	mov	a, #01001100b
	acall	xfer_byte
	mov	a, r2
	rr	a
	rr	a
	anl	a, #00011111b
	acall	xfer_byte
	mov	a, r2
	rr	a
	rr	a
	anl	a, #11000000b
	acall	xfer_byte
	acall	xfer_byte
	acall	delay_10ms
	acall	delay_10ms
	acall	delay_3ms
	acall	delay_3ms
	mov	a, #'.'
	lcall	cout
	sjmp	write_chip_next_page
write_chip_skip_page:
	mov	a, dpl
	add	a, #128
	mov	dpl, a
	jnc	write_chip_next_page
	inc	dph
write_chip_next_page:
	inc	r2
	cjne	r2, #(chip_size / 128), write_chip_page_loop
	lcall	newline
	ret



is_data_to_write:
	push	dph
	push	dpl
	mov	r5, #128
idtw_loop:
	movx	a, @dptr
	cjne	a, #0xFF, idtw_yes
	inc	dptr
	djnz	r5, idtw_loop
	clr	c
	pop	dpl
	pop	dph
	ret
idtw_yes:
	setb	c
	pop	dpl
	pop	dph
	ret















read_chip:
	mov	dptr, #msg_reading
	lcall	pstr
	mov	dptr, #buffer_location
	mov	r2, #0		;r2/r3 to count word address
	mov	r3, #0
read_chip_loop:
	acall	rd_lsb
	movx	@dptr, a
	inc	dptr
	acall	rd_msb
	movx	@dptr, a
	inc	dptr
	acall	inc_r3r2
	cjne	r2, #(chip_size / 2) & 255, read_chip_loop
	mov	a, #'.'
	lcall	cout
	cjne	r3, #(chip_size / 2) >> 8, read_chip_loop
	lcall	newline
	ret



verify_chip:
	mov	dptr, #msg_verifying
	lcall	pstr
	mov	dptr, #buffer_location
	mov	r2, #0		;r2/r3 to count word address
	mov	r3, #0
verify_chip_loop:
	acall	rd_lsb
	mov	b, a
	movx	a, @dptr
	cjne	a, b, verify_chip_error
	inc	dptr
	acall	rd_msb
	mov	b, a
	movx	a, @dptr
	cjne	a, b, verify_chip_error
	inc	dptr
	acall	inc_r3r2
	cjne	r2, #(chip_size / 2) & 255, verify_chip_loop
	mov	a, #'.'
	lcall	cout
	cjne	r3, #(chip_size / 2) >> 8, verify_chip_loop
	mov	dptr, #msg_ok
	lcall	pstr
	ret
verify_chip_error:
	mov	dptr, #msg_error
	lcall	pstr
	ret









erase_chip:
	mov	dptr, #msg_erase
	lcall	pstr
	mov	a, #10101100b
	acall	xfer_byte
	mov	a, #10000000b
	acall	xfer_byte
	acall	xfer_byte
	acall	xfer_byte
	acall	delay_10ms
	acall	delay_10ms
	mov	r2, #0		;r2/r3 to count word address
	mov	r3, #0
erase_verify_loop:
	acall	rd_lsb
	cjne	a, #0xFF, erase_verify_error
	acall	rd_msb
	cjne	a, #0xFF, erase_verify_error
	acall	inc_r3r2
	cjne	r2, #(chip_size / 2) & 255, erase_verify_loop
	mov	a, #'.'
	lcall	cout
	cjne	r3, #(chip_size / 2) >> 8, erase_verify_loop
	mov	dptr, #msg_ok
	lcall	pstr
	ret
erase_verify_error:
	mov	dptr, #msg_error
	lcall	pstr
	ret




set_fuses:
	mov	dptr, #msg_fuses
	lcall	pstr
	mov	a, #10101100b
	acall	xfer_byte
	mov	a, #10100000b
	acall	xfer_byte
	clr	a
	acall	xfer_byte
	mov	a, #fuse_lsb
	acall	xfer_byte
	acall	delay_10ms
	mov	a, #10101100b
	acall	xfer_byte
	mov	a, #10101000b
	acall	xfer_byte
	clr	a
	acall	xfer_byte
	mov	a, #fuse_msb
	acall	xfer_byte
	acall	delay_10ms
fuses_verify:
	mov	a, #01010000b
	acall	xfer_byte
	mov	a, #00000000b
	acall	xfer_byte
	acall	xfer_byte
	acall	xfer_byte
	cjne	a, #fuse_lsb, fuses_error
	mov	a, #01011000b
	acall	xfer_byte
	mov	a, #00001000b
	acall	xfer_byte
	clr	a
	acall	xfer_byte
	clr	a
	acall	xfer_byte
	cjne	a, #fuse_msb, fuses_error
	mov	dptr, #msg_ok
	lcall	pstr
	ret
fuses_error:
	mov	dptr, #msg_error
	lcall	pstr
	ret










programming_enable:
	mov	a, #10101100b
	acall	xfer_byte
	mov	a, #01010011b
	acall	xfer_byte
	clr	a
	acall	xfer_byte
	push	acc
	clr	a
	acall	xfer_byte
	pop	acc
	ret


read_signature_byte:
	push	acc
	mov	a, #00110000b
	acall	xfer_byte
	clr	a
	acall	xfer_byte
	pop	acc
	anl	a, #00000011b
	acall	xfer_byte
	clr	a
	acall	xfer_byte
	ret




wr_lsb:
	push	acc
	mov	a, #01000000b
	acall	xfer_byte
	clr	a
	acall	xfer_byte
	mov	a, r4		;r4 is offset in page
	acall	xfer_byte
	pop	acc
	ajmp	xfer_byte

wr_msb:
	push	acc
	mov	a, #01001000b
	acall	xfer_byte
	clr	a
	acall	xfer_byte
	mov	a, r4		;r4 is offset in page
	acall	xfer_byte
	pop	acc
	ajmp	xfer_byte



rd_lsb:
	mov	a, #00100000b
	acall	xfer_byte
	mov	a, r3
	acall	xfer_byte
	mov	a, r2
	acall	xfer_byte
	ajmp	xfer_byte

rd_msb:
	mov	a, #00101000b
	acall	xfer_byte
	mov	a, r3
	acall	xfer_byte
	mov	a, r2
	acall	xfer_byte
	ajmp	xfer_byte

inc_r3r2:
	inc	r2
	cjne	r2, #0, ir3r2d
	inc	r3
ir3r2d:	ret





enter_pgm_mode:
	setb	txd_en_pin
	clr	buf_en_pin
	clr	sclk_pin
	setb	rst_pin
	setb	mosi_pin
	setb	miso_pin
	acall	delay_3ms
	clr	sclk_pin
	nop
	clr	rst_pin
	acall	delay_10ms
	acall	delay_10ms
	acall	delay_10ms
	acall	delay_10ms
	acall	delay_10ms
	acall	delay_10ms
	acall	delay_10ms
	acall	delay_10ms
	ret



xfer_byte:
	rlc	a
	mov	mosi_pin, c
	 ;acall	dly
	setb	sclk_pin
	mov	c, miso_pin
	rlc	a
	 ;acall	dly
	clr	sclk_pin
	mov	mosi_pin, c
	 ;acall	dly
	setb	sclk_pin
	mov	c, miso_pin
	rlc	a
	 ;acall	dly
	clr	sclk_pin
	mov	mosi_pin, c
	 ;acall	dly
	setb	sclk_pin
	mov	c, miso_pin
	rlc	a
	 ;acall	dly
	clr	sclk_pin
	mov	mosi_pin, c
	 ;acall	dly
	setb	sclk_pin
	mov	c, miso_pin
	rlc	a
	 ;acall	dly
	clr	sclk_pin
	mov	mosi_pin, c
	 ;acall	dly
	setb	sclk_pin
	mov	c, miso_pin
	rlc	a
	 ;acall	dly
	clr	sclk_pin
	mov	mosi_pin, c
	 ;acall	dly
	setb	sclk_pin
	mov	c, miso_pin
	rlc	a
	 ;acall	dly
	clr	sclk_pin
	mov	mosi_pin, c
	 ;acall	dly
	setb	sclk_pin
	mov	c, miso_pin
	rlc	a
	 ;acall	dly
	clr	sclk_pin
	mov	mosi_pin, c
	 ;acall	dly
	setb	sclk_pin
	mov	c, miso_pin
	rlc	a
	 ;acall	dly
	clr	sclk_pin
	ret


dly:	ret


delay_10ms:             ;18432 cycles at 22.1184 MHz
        mov     r0, #36
dly10a: mov     r1, #0
        djnz    r1, *
        djnz    r0, dly10a
        ret


delay_3ms:              ;5530 cycles at 22.1184 MHz
        mov     r0, #79
dly3a:  mov     r1, #34
        djnz    r1, *
        nop
        djnz    r0, dly3a
        ret



mesg_menu:
        .db     13,10,15
        .db     "AVR-Mega16: 'r'=Read, 'w'=Write, 'v'=Verify, 'e'=Erase, 'b'=Boot, 'q'=Quit",13,10,0
msg_reset:
        .db     "Resetting AVR....",13,10,0
msg_boot:
	.db	"Booting AVR Mega16... (press ESC to halt)",13,10,13,10,0
msg_reading:
        .db     "Reading:   ",0
msg_writing:
        .db     "Writing:   ",0
msg_verifying:
        .db     "Verifying: ",0
msg_fuses:
	.db	"Set Fuses: ",0
msg_ck_fuses:
	.db	"Ck Fuses:  ",0
msg_erase:
        .db     "Erasing:   ",0
msg_error:
        .db     "Error",13,10,0
msg_ok:
        .db     "Ok",13,10,0

msg_startup_error:
	.db	"Error: unable to communicate with AVR chip, code=",0
msg_startup_error_2:
	.db	"Perhaps the power is off or the player is in shutdown mode?",0
msg_device_error:
	.db	"Error: signature does not match AVR Mega16",13,10,13,10,0




.org    location+0x600
.db     0xA5,0xE5,0xE0,0xA5     ;signiture bytes
.db     253,255,0,0             ;id (254=cmd)
.db     0,0,0,0                 ;prompt code vector
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;user defined
.db     255,255,255,255         ;length and checksum (255=unused)
.db     "Clear Code Buffer",0
.org    location+0x600+64       ;executable code begins here


        mov     dptr, #buffer_location
	mov	a, dph
	jb	acc.7, clr_end		;don't clear if in flash
        mov     a, #255
clr_loop:
        movx    @dptr, a
        inc     dptr
        mov     r0, dpl
        cjne    r0, #(buffer_location + chip_size) & 255, clr_loop
        mov     r0, dph
        cjne    r0, #(buffer_location + chip_size) >> 8, clr_loop
clr_end:
        ret



                                                                                
.org    location+0xF00
.db     0xA5,0xE5,0xE0,0xA5     ;signiture bytes
.db     249,255,0,0             ;id (249=init)
.db     0,0,0,0                 ;prompt code vector
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;reserved
.db     0,0,0,0                 ;user defined
.db     255,255,255,255         ;length and checksum (255=unused)
.db     "Fixed Baud Rate",0
.org    location+0xF00+64         ;executable code begins here
                                                                                
        mov     a, #baud_const
        mov     0x7B, a
        mov     0x7A, a         ;store the baud rate for next warm boot.
        mov     0x79, a
        mov     0x78, a
        xrl     0x7A, #01010101b
        xrl     0x79, #11001100b
        xrl     0x78, #00011101b
        ret



